=begin
 Copyright 2009, Chris Glasier
# Release 1.0 - Basic configuration
#Release 1.01 - Revisions
#       Added sketchup.rb (thanks to Tomasz SCF)
#       Add new scene setup for new files
#       Rename layer
#Release 1.04
#       skp functions renamed according to DeviceNameAction (DNA!) - e.g. sceneLayersAdd
Release 1.05
Observer and global varaible renamed
START control if no Sketchup.active_model when the Ruby scripts are first loaded.
Code rewritten so that no window.location call is followed by any javascript statements.

Changes since:

added scene set() to start to stop rare "slidersTop[0] = undefined" - e.g. if resizing with only start in display
general tidy up of indentation.
Automatic focusing of scene and component reels after selection in SU display 


SUGGESTIONS FOR SCENE MANAGEMENT
Use cgScenes
left column to browse (1st click) and (2nd click) open scene and current aspect
middle column to open aspects for current scene
middle column layer menu to sort layers by abc or visibility and to add and name new layer
middle column components menu to show or hide all current scene components
right column to browse (1st click) and (2nd click) hide/show properties to save,layers and components, and change views and styles.
Click Update to save scene properties. Update takes priority over Synch.
Click Synch to update after making changes or component selection in Sketchup display.
Use Sketchup
context menu to add, delete, rename or describe a scene
layer window to rename or delete existing layer
layer window to select layer colors
shadows toolbar
If a scene is not showing what is expected double check properties to ensure everything required is saved and check layer and components status as these can contradict eackh other. If still not performing correctly email chriglasier@gmail.com
=end

require 'sketchup.rb'

# I like to see the menu at the top so I know where the main entry point
# for the script is. -- Jim

if (not file_loaded?("chrisglasierScenes01.rb"))
    UI.menu("Plugins").add_item("cgScenes" ){ CgScenes.start }
end



class CgScenesPagesObserver < Sketchup::PagesObserver
    # 
    # Called when a new Scene is added from SketchUp. Jim
	# Called when a new Scene is selected from SketchUp. Chris
    def onContentsModified(pages)
	puts "Modified" #calls twice
	CgScenes.page_contents_modified(pages)
    end
end



class CgScenesSelectionObserver < Sketchup::SelectionObserver
    # 
    attr_reader :selected
    def initialize
        @selected = nil
    end
    def onSelectionBulkChange(selection)
        a = 0
        entities = Sketchup.active_model.entities
        entities.each do |entity|
            if entity.class == Sketchup::Group || entity.class == Sketchup::ComponentInstance
                if selection[0] == entity
                    @selected  = a
                end
            end 
            a += 1
        end  # loop
        CgScenes.selection_bulk_change(selected)
    end  # def
end  # class





module CgScenes

	def CgScenes.page_contents_modified(pages)
        @dlg.execute_script("sceneScenesSelected();")
    end

    def CgScenes.selection_bulk_change(selected)
        @dlg.execute_script("sceneComponentsSelected('#{selected}');")
    end

    def CgScenes.start
		@cgScenesPagesObserver ||= CgScenesPagesObserver.new
		@cgScenesSelectionObserver ||= CgScenesSelectionObserver.new
 
		Sketchup.active_model.pages.add_observer(@cgScenesPagesObserver)
        Sketchup.active_model.selection.add_observer(@cgScenesSelectionObserver)

        Sketchup.send_action "selectSelectionTool:"

        @dlg = UI::WebDialog.new("Scene machine", true, "Scene machine", 230, 345, 0, 0, true)
        subDir = "chrisglasierScenes/"
        fileName = "scene machine.htm"
        @dlg.set_file File.join(File.dirname(__FILE__), subDir+fileName)

        @dlg.add_action_callback("coreSet_size") {|d, p|
            a = p.split(",")
            w = Integer(a[0])
            h = Integer(a[1])
            @dlg.set_size w,h   
        }
        ########## SCENES ##########
        @dlg.add_action_callback("sceneStart") {|d, p|
            model = Sketchup.active_model
            pages = model.pages
            count = pages.count
            if count == 0
                cmd = "sceneScenesAdd();"
            else
                cmd = "sceneScenesRun();"
            end
            @dlg.execute_script (cmd)     
        }
        ##### SCENE ADD
        @dlg.add_action_callback("sceneScenesAdd") {|d, p|
            model = Sketchup.active_model
            pages = model.pages
            pages.add p
            cmd = "sceneScenesAdd();"
            @dlg.execute_script (cmd)
        }
        ##### OPENING SCENE FOR SETTING TUMBLER START
        @dlg.add_action_callback("sceneScenesSelected") {|d, p|
            model = Sketchup.active_model
            pages = model.pages
            a=0
            pages.each do | page |
                if page == pages.selected_page
				    cmd = "sceneScenesSelectedReturn('#{a}');"
                    @dlg.execute_script(cmd)
                end
            a += 1
            end
        }
        ##### SCENE LIST
        @dlg.add_action_callback("sceneScenesList") {|d, p|
            model = Sketchup.active_model
            pages = model.pages
            a = 0
            array = []
            pages.each do | page |
                array[a] = []
            array[a][0] = page.name
            array[a][1] = a
            array[a] = array[a].join(",")
            a+=1
            end  #loop
            array = array.join(";")
            p array
            cmd = "sceneScenesListRun('#{array}');"
            @dlg.execute_script (cmd)
        }
        ##### GOTO SCENE
        @dlg.add_action_callback("sceneSceneOpen") {|d, p|
            a = p.split(",")
            ind = Integer(a[0])
            model = Sketchup.active_model
            pages = model.pages
            pages.selected_page = pages[ind]
            cmd = "sceneSceneOpenRun();"
            @dlg.execute_script (cmd)
        }
        ########## PROPERTIES (to save) ##########
        ##### PROPERTY LIST
        @dlg.add_action_callback("scenePropertiesList") {|d, p|
            model = Sketchup.active_model
            page = model.pages.selected_page
            array = []
            array.push ["Axes",page.use_axes?].join(",")
            array.push ["Camera",page.use_camera?].join(",")
            array.push ["Hidden",page.use_hidden?].join(",")
            array.push ["Hidden layers",page.use_hidden_layers?].join(",")
            array.push ["Rendering options",page.use_rendering_options?].join(",")
            array.push ["Section planes",page.use_section_planes?].join(",")
            array.push ["Shadow info",page.use_shadow_info?].join(",")
            array.push ["Style",page.use_style?].join(",")
            array = array.join(";")
            cmd = "scenePropertiesListRun('#{array}');"
            @dlg.execute_script (cmd)       
        }
        ##### PROPERTY TOGGLES ON/OFF
        @dlg.add_action_callback("scenePropertyToggle") { |d, a|
            #puts "sceneProprtyToggle:#{a.inspect}"
            property, val = a.split(',')
            property.downcase!
            val = ( val == "off" ? true : false )
            model = Sketchup.active_model
            page = model.pages.selected_page
            message = "use_" + property + "="
            #p message, val
            page.__send__(message, val)
        }
        ########## LAYERS ##########
        ##### LAYERS LIST
        @dlg.add_action_callback("sceneLayersList") {|d, p|
            model = Sketchup.active_model
            layers = model.layers
            a = 0
            b=-1
            array = []
            layers.each do |layer|
                b += 1
                array[a] = []
                array[a][0] = layer.name
                array[a][1] = layer.visible?
                array[a][2] = b
                array[a] = array[a].join(",")
                a+=1
            end  #loop
            array = array.join(";")
            cmd = "sceneLayersListRun('#{array}');"
            @dlg.execute_script (cmd)       
        }
        ##### LAYER ADD
        @dlg.add_action_callback("sceneLayersAdd") {|d, p|
            model = Sketchup.active_model
            layers = model.layers
            layers.add "Aa"
            cmd = "sceneLayerRename();"
            @dlg.execute_script (cmd)
        }
        ##### LAYER TOGGLE ON/OFF
        @dlg.add_action_callback("sceneLayerToggleon") {|d, p|
            a= p.split(",")
            ind = Integer(a[0])
            model = Sketchup.active_model
            layer = model.layers[ind]
            layer.visible = false    
        }
        @dlg.add_action_callback("sceneLayerToggleoff") {|d, p|
            a= p.split(",")
            ind = Integer(a[0])
            model = Sketchup.active_model
            layer = model.layers[ind]
            layer.visible = true    
        }
        ##### LAYER RENAME
        @dlg.add_action_callback("sceneLayersRename") {|d, p|
            a = p.split(",")
            newName = a[0].to_str
            model = Sketchup.active_model
            ind = model.layers.length-1
            layer = model.layers[ind]
            layer.name = newName
            cmd = "sceneSynchro();"
            @dlg.execute_script (cmd)
        }
        ########## COMPONENTS (outliner) ##########
        ##### COMPONENT LIST
        @dlg.add_action_callback("sceneComponentsList") {|d, p|
            model = Sketchup.active_model
            entities = model.active_entities
            a = 0
            b = -1
            array = []
            entities.each do |entity|
                b += 1
                if  entity.class == Sketchup::Group || entity.class == Sketchup::ComponentInstance
                    #if  entity.class == Sketchup::ComponentInstance
                    if entity.layer.visible?
                        array[a] = []
                        if  entity.class == Sketchup::ComponentInstance
                            array[a][0] = entity.definition.name
                        end
                        if  entity.class == Sketchup::Group
                            if entity.name == ''
                                array[a][0] = "Group"  #entity.name
                            else
                                array[a][0] = entity.name
                            end
                        end  # if group
                        array[a][1] = entity.hidden?
                        array[a][2] = b #index in entity array for hiliting
                        array[a] = array[a].join(",")
                        a+=1
                    end  # if class
                end  # if visible
            end  # loop
            array = array.join(";")
            cmd = "sceneComponentsListRun('#{array}');"
            @dlg.execute_script (cmd)      
        }
        ##### COMPONENT SHOW/HIDE ALL
        @dlg.add_action_callback("sceneComponentsToggleAll") {|d, p|
            p = p.to_s
            model = Sketchup.active_model
            entities = model.active_entities
            entities.each do |entity|
                if  entity.class == Sketchup::Group || entity.class == Sketchup::ComponentInstance
                    if entity.layer.visible?
                        entity.hidden = false if p == "Show"
                        entity.hidden = true if p == "Hide"
                    end
                end
            end   
        }
        ##### COMPONENT BOUNDING BOX
        @dlg.add_action_callback("sceneHiliteOn") {|d, p|
            entityNo =  Integer(p)
            model = Sketchup.active_model
            entities = model.active_entities
            entity = entities[entityNo]
            selection = model.selection
            selection.clear
            selection.add entity   
        }
        @dlg.add_action_callback("sceneHiliteClear") {|d, p|
            model = Sketchup.active_model
            selection = model.selection
            selection.clear   
        }
        ##### COMPONENT TOGGLE ON/OFF
        @dlg.add_action_callback("sceneComponentToggleoff") {|d, p|
            a= p.split(",")
            ind = Integer(a[0])
            model = Sketchup.active_model
            entity = model.entities[ind]
            entity.hidden = false    
        }
        @dlg.add_action_callback("sceneComponentToggleon") {|d, p|
            a= p.split(",")
            ind = Integer(a[0])
            model = Sketchup.active_model
            entity = model.entities[ind]
            entity.hidden =  true    
        }
        ########## STYLES
        ##### STYLE LIST
        @dlg.add_action_callback("sceneStylesList") {|d, p|
            model = Sketchup.active_model
            styles = model.styles
            a = 0
            array = []
            styles.each do |style|
                array[a] = []
                array[a][0] = style.name
                array[a][1] = a
                array[a] = array[a].join(",")
                a+=1
            end  #loop
            array = array.join(";")
            model = Sketchup.active_model
            pages = model.pages
            name = pages.selected_page.style.name
            array2 = []
            array2[0] = array
            array2[1] = name
            array =array2.join("qq")
            cmd = "sceneStylesListRun('#{array}');"
            @dlg.execute_script (cmd)       
        }
        ##### SET STYLE
        @dlg.add_action_callback("sceneStyleUse") {|d, p|
            a=p.split(",")
            cind = Integer(a[0])
            model = Sketchup.active_model
            styles = model.styles
            style = styles[cind]
            styles.selected_style = style   
        }

        ########## CAMERA VIEWS
        @dlg.add_action_callback("sceneViewTop") {|d, p| Sketchup.send_action("viewTop:")  }
        @dlg.add_action_callback("sceneViewBottom") {|d, p| Sketchup.send_action("viewBottom:")  }
        @dlg.add_action_callback("sceneViewFront") {|d, p| Sketchup.send_action("viewFront:")  }
        @dlg.add_action_callback("sceneViewBack") {|d, p| Sketchup.send_action("viewBack:")  }
        @dlg.add_action_callback("sceneViewLeft") {|d, p| Sketchup.send_action("viewLeft:")  }
        @dlg.add_action_callback("sceneViewRight") {|d, p| Sketchup.send_action("viewRight:")  }
        @dlg.add_action_callback("sceneViewIsometric") {|d, p| Sketchup.send_action("viewIso:")  }
        @dlg.add_action_callback("sceneViewPerspective") {|d, p| Sketchup.send_action("viewPerspective:")  }
        @dlg.add_action_callback("sceneViewZoomExtents") {|d, p| Sketchup.send_action("viewZoomExtents:")  }

        ########## SCENE UPDATE
        @dlg.add_action_callback("sceneUpdate") {|d, p|
            model = Sketchup.active_model
            page = model.pages.selected_page
            page.update   
        }

        @dlg.show()

    end # def CgScenes.start

end # module CgScenes

file_loaded("chrisglasierScenes01.rb")
